@@ -1,23 +1,23 @@
-(********************************************************************************)
-(*  OASIS: architecture for building OCaml libraries and applications           *)
-(*                                                                              *)
-(*  Copyright (C) 2008-2010, OCamlCore SARL                                     *)
-(*                                                                              *)
-(*  This library is free software; you can redistribute it and/or modify it     *)
-(*  under the terms of the GNU Lesser General Public License as published by    *)
-(*  the Free Software Foundation; either version 2.1 of the License, or (at     *)
-(*  your option) any later version, with the OCaml static compilation           *)
-(*  exception.                                                                  *)
-(*                                                                              *)
-(*  This library is distributed in the hope that it will be useful, but         *)
-(*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  *)
-(*  or FITNESS FOR A PARTICULAR PURPOSE. See the file COPYING for more          *)
-(*  details.                                                                    *)
-(*                                                                              *)
-(*  You should have received a copy of the GNU Lesser General Public License    *)
-(*  along with this library; if not, write to the Free Software Foundation,     *)
-(*  Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA               *)
-(********************************************************************************)
+(******************************************************************************)
+(* OASIS: architecture for building OCaml libraries and applications          *)
+(*                                                                            *)
+(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(*                                                                            *)
+(* This library is free software; you can redistribute it and/or modify it    *)
+(* under the terms of the GNU Lesser General Public License as published by   *)
+(* the Free Software Foundation; either version 2.1 of the License, or (at    *)
+(* your option) any later version, with the OCaml static compilation          *)
+(* exception.                                                                 *)
+(*                                                                            *)
+(* This library is distributed in the hope that it will be useful, but        *)
+(* WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY *)
+(* or FITNESS FOR A PARTICULAR PURPOSE. See the file COPYING for more         *)
+(* details.                                                                   *)
+(*                                                                            *)
+(* You should have received a copy of the GNU Lesser General Public License   *)
+(* along with this library; if not, write to the Free Software Foundation,    *)
+(* Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA              *)
+(******************************************************************************)
 
 (** Helper function to write an _oasis file
     @author Sylvain Le Gall
@@ -30,7 +30,6 @@ open Format
 open FormatExt
 open OASISUtils
 open OASISMessage
-open ExtString
 
 type 'a default =
   | Default_value of 'a
@@ -138,7 +137,7 @@ let ask_until_correct t =
 
         | e -> 
             begin
-              Printf.printf (f_ "Error: %s\n") (string_of_exception e);
+              Printf.printf (f_ "Error: %s\n") (Printexc.to_string e);
               begin
                 match t.interface with
                   | Human ->
@@ -313,7 +312,7 @@ let mk_numbered_choices_multi t choices =
     try 
       t.parse s
     with Failure _ ->
-      failwithf2
+      failwithf
         (f_ "'%s' is not valid in answer '%s'")
         s full
   in
@@ -327,8 +326,8 @@ let mk_numbered_choices_multi t choices =
            (fun s -> s <> "") 
            (List.flatten 
               (List.map 
-                 (fun s -> String.nsplit s ",")
-                 (String.nsplit s " "))))
+                 (fun s -> split_comma s)
+                 (OASISString.nsplit s ' '))))
     in
       String.concat ", " lst
   in
@@ -394,7 +393,7 @@ let ask_schema ~ctxt schema lvl interface plugins =
                 try
                   set s
                 with e ->
-                  failwithf3
+                  failwithf
                     (f_ "Trying to set field '%s' using mandatory value '%s': %s")
                     key s (Printexc.to_string e)
               end
@@ -425,7 +424,7 @@ let ask_schema ~ctxt schema lvl interface plugins =
                     question  = Printf.sprintf 
                                   (f_ "Value for field '%s'?") 
                                   key;
-                    parse     = (fun s -> s);
+                    parse     = (fun s -> set s; s);
                   }
                 in
                   try 
@@ -558,6 +557,8 @@ let ask_package ~ctxt lvl intrf =
       OASISPlugin.SetPlugin.empty
   in
 
+  let oasis_version = OASISConf.version_short in
+
   let mk_t nm hlp q = 
     {
       id        = nm;
@@ -589,7 +590,7 @@ let ask_package ~ctxt lvl intrf =
       let data, _ = 
         ask_schema ~ctxt schema lvl intrf plugins
       in
-        gen nm data
+        gen oasis_version nm data
     in
 
     let sections = 
@@ -648,7 +649,7 @@ let ask_package ~ctxt lvl intrf =
                 with 
                     parse = 
                       (fun s -> 
-                         failwithf1 
+                         failwithf
                            (f_ "'%s' is not a valid choice")
                            s);
                     default = Default_value None}
@@ -717,7 +718,7 @@ let ask_package ~ctxt lvl intrf =
     pkg
 
 (** Create an _oasis file *)
-let to_file ~ctxt fn lvl intrf oasis_dev =
+let to_file ~ctxt fn lvl intrf oasis_setup =
 
   let () = 
     (* Print introduction *)
@@ -764,7 +765,7 @@ let to_file ~ctxt fn lvl intrf oasis_dev =
               })
         in
           if not a then
-            failwithf1
+            failwithf
               (f_ "File '%s' already exists, remove it first")
               fn
       end
@@ -802,7 +803,7 @@ let to_file ~ctxt fn lvl intrf oasis_dev =
         try 
           begin
             let prg = 
-              FileUtil.which prg
+              OASISFileUtil.which ~ctxt prg
             in
               info 
                 ~ctxt 
@@ -829,7 +830,7 @@ let to_file ~ctxt fn lvl intrf oasis_dev =
       match Sys.command cmd with
         | 0 -> ()
         | i -> 
-            failwithf2
+            failwithf
               (f_ "Command '%s' exited with status code %d")
               cmd
               i
@@ -871,7 +872,7 @@ let to_file ~ctxt fn lvl intrf oasis_dev =
 
                 with e ->
                   Sys.remove tmp_fn;
-                  error ~ctxt "%s" (string_of_exception e)
+                  error ~ctxt "%s" (Printexc.to_string e)
               end
           end
       | None ->
@@ -890,7 +891,7 @@ let to_file ~ctxt fn lvl intrf oasis_dev =
                 Sys.remove tmp_fn
               with e ->
                 Sys.remove tmp_fn;
-                error ~ctxt "%s" (string_of_exception e)
+                error ~ctxt "%s" (Printexc.to_string e)
           end
       | None ->
           begin
@@ -920,7 +921,7 @@ let to_file ~ctxt fn lvl intrf oasis_dev =
            question  = "What do you want to do now?";
            parse = 
              (fun s -> 
-                failwithf1 
+                failwithf
                   (f_ "'%s' is not a valid choice.")
                   s);
          }
@@ -942,10 +943,10 @@ let to_file ~ctxt fn lvl intrf oasis_dev =
               create_fn content fn;
               false);
 
-           s_ "r", s_ "write, run 'oasis setup-dev' and exit", None,
+           s_ "r", s_ "write, run 'oasis setup' and exit", None,
            (fun () ->
               create_fn content fn;
-              oasis_dev ();
+              oasis_setup ();
               false);
            
            s_ "q", s_ "exit without saving", None,
