@@ -1,23 +1,23 @@
-(********************************************************************************)
-(*  OASIS: architecture for building OCaml libraries and applications           *)
-(*                                                                              *)
-(*  Copyright (C) 2008-2010, OCamlCore SARL                                     *)
-(*                                                                              *)
-(*  This library is free software; you can redistribute it and/or modify it     *)
-(*  under the terms of the GNU Lesser General Public License as published by    *)
-(*  the Free Software Foundation; either version 2.1 of the License, or (at     *)
-(*  your option) any later version, with the OCaml static compilation           *)
-(*  exception.                                                                  *)
-(*                                                                              *)
-(*  This library is distributed in the hope that it will be useful, but         *)
-(*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  *)
-(*  or FITNESS FOR A PARTICULAR PURPOSE. See the file COPYING for more          *)
-(*  details.                                                                    *)
-(*                                                                              *)
-(*  You should have received a copy of the GNU Lesser General Public License    *)
-(*  along with this library; if not, write to the Free Software Foundation,     *)
-(*  Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA               *)
-(********************************************************************************)
+(******************************************************************************)
+(* OASIS: architecture for building OCaml libraries and applications          *)
+(*                                                                            *)
+(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(*                                                                            *)
+(* This library is free software; you can redistribute it and/or modify it    *)
+(* under the terms of the GNU Lesser General Public License as published by   *)
+(* the Free Software Foundation; either version 2.1 of the License, or (at    *)
+(* your option) any later version, with the OCaml static compilation          *)
+(* exception.                                                                 *)
+(*                                                                            *)
+(* This library is distributed in the hope that it will be useful, but        *)
+(* WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY *)
+(* or FITNESS FOR A PARTICULAR PURPOSE. See the file COPYING for more         *)
+(* details.                                                                   *)
+(*                                                                            *)
+(* You should have received a copy of the GNU Lesser General Public License   *)
+(* along with this library; if not, write to the Free Software Foundation,    *)
+(* Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA              *)
+(******************************************************************************)
 
 (** Functions common to OCamlbuild build and doc plugin
   *)
@@ -33,7 +33,7 @@ let ocamlbuildflags =
   var_define
     ~short_desc:(fun () -> "OCamlbuild additional flags")
     "ocamlbuildflags"
-    (lazy "")
+    (fun () -> "")
 
 (** Fix special arguments depending on environment *)
 let fix_args args extra_argv =
@@ -41,18 +41,18 @@ let fix_args args extra_argv =
     [
       if (os_type ()) = "Win32" then
         [
-          "-classic-display"; 
-          "-no-log"; 
+          "-classic-display";
+          "-no-log";
           "-no-links";
-          "-install-lib-dir"; 
+          "-install-lib-dir";
           (Filename.concat (standard_library ()) "ocamlbuild")
-        ] 
+        ]
       else
         [];
-  
+
       if not (bool_of_string (is_native ())) || (os_type ()) = "Win32" then
         [
-          "-byte-plugin" 
+          "-byte-plugin"
         ]
       else
         [];
@@ -68,7 +68,7 @@ let fix_args args extra_argv =
       else
         [];
 
-      OASISUtils.split ' ' (ocamlbuildflags ());
+      OASISString.nsplit (ocamlbuildflags ()) ' ';
 
       Array.to_list extra_argv;
     ]
@@ -81,11 +81,12 @@ let run_clean extra_argv =
     (* Run if never called with these args *)
     if not (BaseLog.exists ocamlbuild_clean_ev extra_cli) then
       begin
-        BaseExec.run (ocamlbuild ()) (fix_args ["-clean"] extra_argv);
+        OASISExec.run ~ctxt:!BaseContext.default
+          (ocamlbuild ()) (fix_args ["-clean"] extra_argv);
         BaseLog.register ocamlbuild_clean_ev extra_cli;
-        at_exit 
+        at_exit
           (fun () ->
-             try 
+             try
                BaseLog.unregister ocamlbuild_clean_ev extra_cli
              with _ ->
                ())
@@ -95,7 +96,8 @@ let run_clean extra_argv =
 let run_ocamlbuild args extra_argv =
   (* TODO: enforce that target in args must be UNIX encoded i.e. toto/index.html
    *)
-  BaseExec.run (ocamlbuild ()) (fix_args args extra_argv);
+  OASISExec.run ~ctxt:!BaseContext.default
+    (ocamlbuild ()) (fix_args args extra_argv);
   (* Remove any clean event, we must run it again *)
   List.iter
     (fun (e, d) -> BaseLog.unregister e d)
@@ -109,7 +111,7 @@ let build_dir extra_argv =
           search_args dir tl
       | _ :: tl ->
           search_args dir tl
-      | [] -> 
+      | [] ->
           dir
   in
     search_args "_build" (fix_args [] extra_argv)
@@ -118,26 +120,26 @@ let build_dir extra_argv =
 
 open OASISTypes
 
-let fix_build_tools tool pkg = 
-  let fix_build_tools' sct bs = 
+let fix_build_tools tool pkg =
+  let fix_build_tools' sct bs =
     if not (List.mem tool bs.bs_build_tools) then
       {bs with bs_build_tools = tool :: bs.bs_build_tools}
     else
       bs
   in
 
-  let sections = 
+  let sections =
     List.fold_left
       (fun acc sct ->
-         let sct = 
-           match sct with 
+         let sct =
+           match sct with
              | Executable (cs, bs, exec) ->
                  let bs = fix_build_tools' sct bs in
                    Executable (cs, bs, exec)
 
              | Library (cs, bs, lib) ->
                  let bs = fix_build_tools' sct bs in
-                   Library (cs, bs, lib) 
+                   Library (cs, bs, lib)
 
              | Flag _ | SrcRepo _ | Test _ | Doc _ as sct ->
                  sct
@@ -149,15 +151,12 @@ let fix_build_tools tool pkg =
     {pkg with sections = List.rev sections}
 
 
-module Tag = 
-struct 
-  (** [filename_concat fn1 fn2] Concat filename, using semantic of _tags 
+module Tag =
+struct
+  (** [filename_concat fn1 fn2] Concat filename, using semantic of _tags
       [fn1] must be a real filename whereas fn2 can contains wildcards.
     *)
-  let filename_concat fn1 fn2 = 
-    (* TODO: consider using directly ocamlbuild function *)
-    FilePath.UnixPath.concat 
-      (FilePath.UnixPath.reduce ~no_symlink:true fn1)
-      fn2 
+  let filename_concat fn1 fn2 =
+    OASISUnixPath.concat (OASISUnixPath.reduce fn1) fn2
 
 end
