@@ -1,25 +1,25 @@
-(********************************************************************************)
-(*  OASIS: architecture for building OCaml libraries and applications           *)
-(*                                                                              *)
-(*  Copyright (C) 2008-2010, OCamlCore SARL                                     *)
-(*                                                                              *)
-(*  This library is free software; you can redistribute it and/or modify it     *)
-(*  under the terms of the GNU Lesser General Public License as published by    *)
-(*  the Free Software Foundation; either version 2.1 of the License, or (at     *)
-(*  your option) any later version, with the OCaml static compilation           *)
-(*  exception.                                                                  *)
-(*                                                                              *)
-(*  This library is distributed in the hope that it will be useful, but         *)
-(*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  *)
-(*  or FITNESS FOR A PARTICULAR PURPOSE. See the file COPYING for more          *)
-(*  details.                                                                    *)
-(*                                                                              *)
-(*  You should have received a copy of the GNU Lesser General Public License    *)
-(*  along with this library; if not, write to the Free Software Foundation,     *)
-(*  Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA               *)
-(********************************************************************************)
+(******************************************************************************)
+(* OASIS: architecture for building OCaml libraries and applications          *)
+(*                                                                            *)
+(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(*                                                                            *)
+(* This library is free software; you can redistribute it and/or modify it    *)
+(* under the terms of the GNU Lesser General Public License as published by   *)
+(* the Free Software Foundation; either version 2.1 of the License, or (at    *)
+(* your option) any later version, with the OCaml static compilation          *)
+(* exception.                                                                 *)
+(*                                                                            *)
+(* This library is distributed in the hope that it will be useful, but        *)
+(* WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY *)
+(* or FITNESS FOR A PARTICULAR PURPOSE. See the file COPYING for more         *)
+(* details.                                                                   *)
+(*                                                                            *)
+(* You should have received a copy of the GNU Lesser General Public License   *)
+(* along with this library; if not, write to the Free Software Foundation,    *)
+(* Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA              *)
+(******************************************************************************)
 
-(** Build using ocamlbuild  
+(** Build using ocamlbuild
     @author Sylvain Le Gall
   *)
 
@@ -31,10 +31,6 @@ open OCamlbuildCommon
 open BaseStandardVar
 open BaseMessage
 
-type target =
-  | Std of string list
-  | StdRename of string * string
-
 let cond_targets_hook =
   ref (fun lst -> lst)
 
@@ -42,24 +38,24 @@ let build pkg argv =
 
   (* Return the filename in build directory *)
   let in_build_dir fn =
-    Filename.concat 
-      (build_dir argv) 
+    Filename.concat
+      (build_dir argv)
       fn
   in
 
   (* Return the unix filename in host build directory *)
   let in_build_dir_of_unix fn =
-    in_build_dir (BaseFilePath.of_unix fn)
+    in_build_dir (OASISHostPath.of_unix fn)
   in
 
   let cond_targets =
     List.fold_left
       (fun acc ->
-         function 
+         function
            | Library (cs, bs, lib) when var_choose bs.bs_build ->
                begin
                  let evs, unix_files =
-                   BaseBuilt.of_library 
+                   BaseBuilt.of_library
                      in_build_dir_of_unix
                      (cs, bs, lib)
                  in
@@ -70,65 +66,62 @@ let build pkg argv =
                    in
                      (String.length fn >= nd_len)
                      &&
-                     (String.sub 
+                     (String.sub
                         fn
                         (String.length fn - nd_len)
                         nd_len) = nd
                  in
 
                  let tgts =
-                   List.filter
-                     (fun l -> l <> [])
-                     (List.map
-                        (List.filter
-                           (fun fn ->
-                              ends_with ".cma" fn ||
-                              ends_with ".cmxa" fn ||
-                              ends_with (ext_lib ()) fn ||
-                              ends_with (ext_dll ()) fn))
-                        unix_files)
+                   List.flatten
+                     (List.filter
+                        (fun l -> l <> [])
+                        (List.map
+                           (List.filter
+                              (fun fn ->
+                               ends_with ".cma" fn
+                               || ends_with ".cmxs" fn
+                               || ends_with ".cmxa" fn
+                               || ends_with (ext_lib ()) fn
+                               || ends_with (ext_dll ()) fn))
+                           unix_files))
                  in
 
-                   match tgts with 
-                     | hd :: tl ->
-                         (evs, Std hd)
-                         :: 
-                         (List.map (fun tgts -> [], Std tgts) tl)
-                         @
-                         acc
+                   match tgts with
+                     | _ :: _ ->
+                         (evs, tgts) :: acc
                      | [] ->
-                         failwithf2
-                           (f_ "No possible ocamlbuild targets \
-                                in generated files %s for library %s")
-                           (String.concat (s_ ", " ) (List.map (String.concat (s_ ", ")) tgts))
+                         failwithf
+                           (f_ "No possible ocamlbuild targets for library %s")
                            cs.cs_name
                end
 
            | Executable (cs, bs, exec) when var_choose bs.bs_build ->
                begin
                  let evs, unix_exec_is, unix_dll_opt =
-                   BaseBuilt.of_executable 
+                   BaseBuilt.of_executable
                      in_build_dir_of_unix
                      (cs, bs, exec)
                  in
 
-                 let host_exec_is = 
-                   in_build_dir_of_unix unix_exec_is
-                 in
-
                  let target ext =
-                   let unix_tgt = 
-                     (BaseFilePath.Unix.concat
+                   let unix_tgt =
+                     (OASISUnixPath.concat
                         bs.bs_path
-                        (BaseFilePath.Unix.chop_extension 
+                        (OASISUnixPath.chop_extension
                            exec.exec_main_is))^ext
                    in
-
-                     evs,
-                     (if unix_tgt = unix_exec_is then
-                        Std [unix_tgt]
-                      else
-                        StdRename (unix_tgt, host_exec_is))
+                   let evs = 
+                     (* Fix evs, we want to use the unix_tgt, without copying *)
+                     List.map
+                       (function
+                          | BaseBuilt.BExec, nm, lst when nm = cs.cs_name ->
+                              BaseBuilt.BExec, nm, [[in_build_dir_of_unix unix_tgt]]
+                          | ev ->
+                              ev)
+                       evs
+                   in
+                     evs, [unix_tgt]
                  in
 
                  (* Add executable *)
@@ -145,7 +138,7 @@ let build pkg argv =
                    acc
                end
 
-           | Library _ | Executable _ | Test _ 
+           | Library _ | Executable _ | Test _
            | SrcRepo _ | Flag _ | Doc _ ->
                acc)
       []
@@ -154,97 +147,34 @@ let build pkg argv =
   in
 
   (* Check and register built files *)
-  let check_and_register (bt, bnm, lst) = 
+  let check_and_register (bt, bnm, lst) =
     List.iter
       (fun fns ->
-         if not (List.exists Sys.file_exists fns) then
-           failwithf1
+         if not (List.exists OASISFileUtil.file_exists_case fns) then
+           failwithf
              (f_ "No one of expected built files %s exists")
              (String.concat (s_ ", ") (List.map (Printf.sprintf "'%s'") fns)))
       lst;
-      (BaseBuilt.register bt bnm lst) 
+      (BaseBuilt.register bt bnm lst)
   in
 
-  (* Run a list of target + post process *)
-  let run_ocamlbuild rtargets = 
+  let cond_targets =
+    (* Run the hook *)
+    !cond_targets_hook cond_targets
+  in
+
+    (* Run a list of target... *)
     run_ocamlbuild 
-      (List.rev_map snd rtargets)
+      (List.flatten 
+         (List.map snd cond_targets))
       argv;
+    (* ... and register events *)
     List.iter
       check_and_register
-      (List.flatten (List.rev_map fst rtargets))
-  in
+      (List.flatten (List.map fst cond_targets))
 
-  (* Compare two files, return true if they differ *)
-  let diff fn1 fn2 =
-    if Sys.file_exists fn1 && Sys.file_exists fn2 then
-      begin
-        let chn1 = open_in fn1 in
-        let chn2 = open_in fn2 in
-        let res =
-          if in_channel_length chn1 = in_channel_length chn2 then
-            begin
-              let len =
-                4096
-              in
-              let str1 = 
-                String.make len '\000'
-              in
-              let str2 =
-                String.copy str1
-              in
-                try
-                  while (String.compare str1 str2) = 0 do 
-                    really_input chn1 str1 0 len;
-                    really_input chn2 str2 0 len
-                  done;
-                  true
-                with End_of_file ->
-                  false
-            end
-          else
-            true
-        in
-          close_in chn1; close_in chn2;
-          res
-      end
-    else
-      true
-  in
 
-  let last_rtargets =
-    List.fold_left
-      (fun acc (built, tgt) ->
-         match tgt with 
-           | Std nms -> 
-               (built, List.hd nms) :: acc
-           | StdRename (src, tgt) ->
-               begin
-                 (* We run with a fake list for event registering *)
-                 run_ocamlbuild (([], src) :: acc);
-
-                 (* And then copy and register *)
-                 begin
-                   let src_fn = 
-                     in_build_dir_of_unix src 
-                   in
-                     if diff src_fn tgt then
-                       BaseFileUtil.cp src_fn tgt
-                     else
-                       info 
-                         (f_ "No need to copy file '%s' to '%s', same content")
-                         src_fn tgt
-                 end;
-                 List.iter check_and_register built;
-                 []
-               end)
-      []
-      (!cond_targets_hook cond_targets)
-  in
-    if last_rtargets <> [] then
-      run_ocamlbuild last_rtargets
-
-let clean pkg extra_args  = 
+let clean pkg extra_args  =
   run_clean extra_args;
   List.iter
     (function
@@ -269,31 +199,28 @@ open OASISTypes
 open OASISValues
 open MyOCamlbuildBase
 open Ocamlbuild_plugin
-open OCamlbuildId 
+open OCamlbuildId
 
-let plugin = 
+let plugin =
   `Build, name, Some version
 
-let self_id, all_id = 
+let self_id, all_id =
   Build.create plugin
 
-(* TODO: check everywhere that having .h and .c in CSources
- * doesn't disturb things too much 
- *)
-let only_h_files lst = 
-  List.filter 
-    (fun fn -> FilePath.UnixPath.check_extension fn "h")
+let only_h_files lst =
+  List.filter
+    (fun fn -> OASISUnixPath.check_extension fn "h")
     lst
 
 let only_c_files lst =
-  List.filter 
-    (fun fn -> FilePath.UnixPath.check_extension fn "c")
+  List.filter
+    (fun fn -> OASISUnixPath.check_extension fn "c")
     lst
 
-let add_tags tag_t tgts tags = 
+let add_tags tag_t tgts tags =
   let quote_target fn =
     let test_char =
-      String.contains fn 
+      String.contains fn
     in
       if test_char '*' ||
          test_char '?' ||
@@ -318,37 +245,34 @@ let add_tags tag_t tgts tags =
 let prepend_bs_path bs fn =
   Tag.filename_concat bs.bs_path fn
 
-let bs_paths bs files = 
-  let subdirs = 
-    List.rev_map FilePath.UnixPath.dirname
+let bs_paths bs files =
+  let subdirs =
+    List.rev_map OASISUnixPath.dirname
       (List.rev_map (prepend_bs_path bs) files)
   in
     (* Unique elements *)
     SetString.elements
       (set_string_of_list
-         (List.rev_map 
-            (FilePath.UnixPath.reduce ~no_symlink:true)
-            (bs.bs_path :: subdirs)))
+         (List.rev_map OASISUnixPath.reduce (bs.bs_path :: subdirs)))
 
+let bs_tags pkg sct cs bs src_dirs src_internal_dirs link_tgt ctxt tag_t myocamlbuild_t =
 
-let bs_tags pkg sct cs bs src_dirs src_internal_dirs link_tgt ctxt tag_t myocamlbuild_t = 
-
-  let link_pkg = 
+  let link_pkg =
     (* Only link findlib package with executable *)
-    match sct with 
+    match sct with
       | Executable _ -> true
       | Library _ | Flag _ | Test _ | SrcRepo _ | Doc _ -> false
   in
 
-  let src_tgts = 
+  let src_tgts =
     List.rev_append
       (* .ml files *)
-      (List.rev_map 
+      (List.rev_map
          (fun dn -> Tag.filename_concat dn "*.ml{,i}")
          (src_dirs @ src_internal_dirs))
       (* .c files *)
-      (List.map 
-         (prepend_bs_path bs) 
+      (List.map
+         (prepend_bs_path bs)
          (only_c_files bs.bs_c_sources))
   in
 
@@ -358,11 +282,11 @@ let bs_tags pkg sct cs bs src_dirs src_internal_dirs link_tgt ctxt tag_t myocaml
          (fun fmt -> (prepend_bs_path bs) (Printf.sprintf fmt cs.cs_name))
          [
            (* Unix *)
-           "dll%s.so";
-           "lib%s.a";
+           "dll%s_stubs.so";
+           "lib%s_stubs.a";
            (* Win32 *)
-           "dll%s.dll";
-           "lib%s.lib";
+           "dll%s_stubs.dll";
+           "lib%s_stubs.lib";
          ])
     else
       []
@@ -375,8 +299,8 @@ let bs_tags pkg sct cs bs src_dirs src_internal_dirs link_tgt ctxt tag_t myocaml
   in
   let mkspec pre_arg tr_arg lst =
     S (List.fold_left
-         (fun acc arg -> 
-            match pre_arg with 
+         (fun acc arg ->
+            match pre_arg with
               | Some s -> s :: arg :: acc
               | None -> arg :: acc)
          []
@@ -384,31 +308,31 @@ let bs_tags pkg sct cs bs src_dirs src_internal_dirs link_tgt ctxt tag_t myocaml
   in
 
   (* Create flag for extra command line option *)
-  let ctxt, tag_t, myocamlbuild_t = 
+  let ctxt, tag_t, myocamlbuild_t =
     List.fold_left
       (fun ((ctxt, tag_t, myocamlbuild_t) as acc)
              (basename, tgts, tags, pre_arg, tr_arg, args_cond) ->
          if args_cond <> [OASISExpr.EBool true, []] then
            begin
-             let tag_name = 
+             let tag_name =
                (* e.g. oasis_library_foo_ccopt *)
-               varname_concat 
+               varname_concat
                  "oasis_"
                  (varname_concat
-                    (varname_of_string 
+                    (varname_of_string
                        (OASISSection.string_of_section sct))
                     basename)
              in
              let all_tags =
-               tag_name :: tags 
+               tag_name :: tags
              in
-             let cond_specs = 
-               List.map 
+             let cond_specs =
+               List.map
                  (fun (cond, lst) ->
                     cond, mkspec pre_arg tr_arg lst)
                  args_cond
              in
-             let flags = 
+             let flags =
                (all_tags, cond_specs) :: myocamlbuild_t.flags
              in
              let tag_t =
@@ -426,10 +350,10 @@ let bs_tags pkg sct cs bs src_dirs src_internal_dirs link_tgt ctxt tag_t myocaml
       (ctxt, tag_t, myocamlbuild_t)
 
       ([
-        "ccopt", 
+        "ccopt",
         src_tgts, ["compile"],
         Some (A"-ccopt"),
-        atom, 
+        atom,
         bs.bs_ccopt;
 
         "cclib",
@@ -485,8 +409,8 @@ let bs_tags pkg sct cs bs src_dirs src_internal_dirs link_tgt ctxt tag_t myocaml
             ::
             acc)
          []
-         ["compile"; 
-          "ocamldep"; 
+         ["compile";
+          "ocamldep";
           "link"]))
   in
 
@@ -495,35 +419,35 @@ let bs_tags pkg sct cs bs src_dirs src_internal_dirs link_tgt ctxt tag_t myocaml
     if bs.bs_c_sources <> [] then
       begin
         (* Generate .clib files *)
-        let fn_clib = 
-          FilePath.add_extension 
-            (prepend_bs_path bs ("lib"^cs.cs_name))
+        let fn_clib =
+          OASISHostPath.add_extension
+            (prepend_bs_path bs ("lib"^(nm_libstubs cs.cs_name)))
             "clib"
         in
-          add_file 
+          add_file
             (template_make
                fn_clib
                comment_ocamlbuild
                []
-               (List.map 
-                  (fun fn -> FilePath.replace_extension fn "o")
+               (List.map
+                  (fun fn -> (Filename.chop_extension fn)^".o")
                   (only_c_files bs.bs_c_sources))
                [])
             ctxt,
 
-          add_tags 
-            tag_t 
-            [link_tgt] 
-            ["use_lib"^cs.cs_name],
+          add_tags
+            tag_t
+            [link_tgt]
+            [tag_libstubs cs.cs_name],
 
-          {myocamlbuild_t with 
-               lib_c = 
-                 ((cs.cs_name, 
-                   bs.bs_path, 
-                   List.map 
+          {myocamlbuild_t with
+               lib_c =
+                 ((cs.cs_name,
+                   bs.bs_path,
+                   List.map
                      (fun fn -> prepend_bs_path bs fn)
                      (only_h_files bs.bs_c_sources))
-                 :: 
+                 ::
                   myocamlbuild_t.lib_c)}
       end
     else
@@ -535,13 +459,13 @@ let bs_tags pkg sct cs bs src_dirs src_internal_dirs link_tgt ctxt tag_t myocaml
     let mp =
       OASISBuildSection.transitive_build_depends pkg
     in
-      add_tags 
+      add_tags
         tag_t
         (if link_pkg then
            link_tgt :: src_tgts
          else
            src_tgts)
-        (List.fold_left 
+        (List.fold_left
            (fun acc ->
               function
                 | FindlibPackage (findlib_pkg, _) ->
@@ -552,27 +476,11 @@ let bs_tags pkg sct cs bs src_dirs src_internal_dirs link_tgt ctxt tag_t myocaml
            (OASISSection.MapSection.find sct mp))
   in
 
-  (* Fix for PR#5015, unable to compile depends in subdir *)
-  let tag_t =
-    let dirs = 
-      src_dirs @ src_internal_dirs 
-    in
-      (if List.length dirs > 1 then
-         add_tags 
-           tag_t
-           (List.filter 
-              (fun fn -> not (FilePath.UnixPath.is_current fn))
-              dirs)
-           ["include"]
-       else
-         tag_t)
-  in
-
-  let ctxt = 
+  let ctxt =
     (* TODO: merge with qstr_cmplt *)
     set_error
       (not (List.mem (ExternalTool "ocamlbuild") bs.bs_build_tools))
-      (Printf.sprintf 
+      (Printf.sprintf
          (f_ "ocamlbuild in field BuildTools of %s is mandatory")
          (OASISSection.string_of_section sct))
       ctxt
@@ -580,40 +488,161 @@ let bs_tags pkg sct cs bs src_dirs src_internal_dirs link_tgt ctxt tag_t myocaml
 
     ctxt, tag_t, myocamlbuild_t
 
-let add_ocamlbuild_files ctxt pkg = 
+module MapDirs = 
+  Map.Make 
+    (struct 
+       type t = [`Library of string | `Executable of string]
+       let compare t1 t2 = 
+         match t1, t2 with 
+           | `Library _, `Executable _ ->
+               -1
+           | `Executable _, `Library _ ->
+               1
+           | `Library s1, `Library s2 
+           | `Executable s1, `Executable s2 ->
+               String.compare s1 s2
+     end)
 
-  let ctxt, tag_t, myocamlbuild_t = 
-    List.fold_left 
+let compute_map_dirs pkg = 
+  let add k dirs internal_dirs mp =
+    let src_dirs, src_internal_dirs = 
+      try 
+        MapDirs.find k mp
+      with Not_found ->
+        SetString.empty, SetString.empty
+    in
+    let add_dirs st dirs = SetString.union st (set_string_of_list dirs) in
+      MapDirs.add k 
+        (add_dirs src_dirs dirs, 
+         add_dirs src_internal_dirs internal_dirs) 
+        mp
+  in
+  let map_dirs = 
+    List.fold_left
+      (fun mp ->
+         function
+           | Library (cs, bs, lib) ->
+               add (`Library cs.cs_name)
+                 (* All paths accessed from within the library *)
+                 (bs_paths bs lib.lib_modules)
+                 (* All paths accessed only by the library *)
+                 (bs_paths bs lib.lib_internal_modules)
+                 mp
+
+           | Executable (cs, bs, exec) ->
+               add (`Executable cs.cs_name)
+                 (bs_paths bs [exec.exec_main_is])
+                 [] (* No internal paths *)
+                 mp
+           | Flag _ | SrcRepo _ | Test _ | Doc _ ->
+               mp)
+      MapDirs.empty
+      pkg.sections
+  in
+    (* Now get rid of internal paths that are also non internal. *)
+    MapDirs.map
+      (fun (src_dirs, src_internal_dirs) ->
+         SetString.elements src_dirs,
+         SetString.elements (SetString.diff src_internal_dirs src_dirs))
+      map_dirs
+
+let compute_includes map_dirs pkg =
+  let add_includes dir set_dirs includes = 
+    (* Not self-dependent *)
+    let set_dirs = SetString.diff set_dirs (SetString.singleton dir) in 
+    let pre_dirs = 
+      try 
+        MapString.find dir includes 
+      with Not_found ->
+        SetString.empty
+    in
+      MapString.add dir (SetString.union set_dirs pre_dirs) includes
+  in
+
+  let add_map_dirs k bs includes = 
+    let dep_dirs = 
+      (* Source dirs of dependent libraries *)
+      List.fold_left 
+        (fun set ->
+           function
+             | InternalLibrary nm ->
+                let src_dirs, _ = 
+                   MapDirs.find (`Library nm) map_dirs 
+                 in
+                   SetString.union set (set_string_of_list src_dirs)
+
+             | FindlibPackage _ ->
+                 set)
+        SetString.empty
+        bs.bs_build_depends
+    in
+    let self_dirs =
+      (* Source dirs *)
+      let src_dirs, src_internal_dirs = MapDirs.find k map_dirs in
+        SetString.union 
+          (set_string_of_list src_dirs)
+          (set_string_of_list src_internal_dirs)
+    in
+    let all_dirs = SetString.union dep_dirs self_dirs in
+    let all_dirs = 
+      (* No need to include the current dir. *)
+      SetString.filter
+        (fun dn -> not (OASISUnixPath.is_current_dir dn))
+        all_dirs
+    in
+        
+      (* All self_dirs depends on all_dirs *)
+      SetString.fold
+        (fun dir includes ->
+           add_includes dir all_dirs includes)
+        self_dirs
+        includes
+  in
+
+  let includes = 
+    List.fold_left
+      (fun includes ->
+         function
+           | Library (cs, bs, _) ->
+               add_map_dirs (`Library cs.cs_name) bs includes
+           | Executable (cs, bs, _) ->
+               add_map_dirs (`Executable cs.cs_name) bs includes
+           | Flag _ | SrcRepo _ | Test _ | Doc _ ->
+               includes)
+      MapString.empty
+      pkg.sections
+  in
+    MapString.fold
+      (fun dir include_dirs acc ->
+         if SetString.empty <> include_dirs then
+           (dir, SetString.elements include_dirs) :: acc
+         else
+           acc)
+      includes
+      []
+
+let add_ocamlbuild_files ctxt pkg =
+
+  let map_dirs = 
+    compute_map_dirs pkg
+  in
+
+  let ctxt, tag_t, myocamlbuild_t =
+    List.fold_left
       (fun (ctxt, tag_t, myocamlbuild_t) ->
-         function 
+         function
            | Library (cs, bs, lib) as sct ->
                begin
                  (* Extract content for libraries *)
 
-                 (* All paths accessed from within the library *)
-                 let src_dirs = 
-                   bs_paths bs lib.lib_modules
-                 in
-
-                 (* All paths accessed only by the library *)
-                 let src_internal_dirs =
-                   let set_dirs = 
-                     set_string_of_list 
-                       src_dirs
-                   in
-                   let set_internal_dirs = 
-                     set_string_of_list
-                       (bs_paths bs lib.lib_internal_modules)
-                   in
-                     SetString.elements
-                       (SetString.diff 
-                          set_internal_dirs
-                          set_dirs)
+                 (* All paths of the library *)
+                 let src_dirs, src_internal_dirs = 
+                   MapDirs.find (`Library cs.cs_name) map_dirs
                  in
 
                  (* Generated library *)
                  let target_lib =
-                   let ext = 
+                   let ext =
                      match bs.bs_compiled_object with
                        | Best ->
                            "{cma,cmxa}"
@@ -623,29 +652,49 @@ let add_ocamlbuild_files ctxt pkg =
                            "cmxa"
                    in
                      prepend_bs_path bs
-                       (FilePath.UnixPath.add_extension
-                          cs.cs_name
-                          ext)
+                       (OASISUnixPath.add_extension cs.cs_name ext)
                  in
 
                  (* Start comment *)
-                 let tag_t = 
+                 let tag_t =
                    (Printf.sprintf "# Library %s" cs.cs_name) :: tag_t
                  in
 
-                 (* Add include tag if the library is internal *)
+                 (* Add dependency of cmxs to their own library: used
+                    at link time when there is C code *)
                  let tag_t =
-                   add_tags 
+                   add_tags
+                     tag_t
+                     [prepend_bs_path
+                        bs
+                        (OASISUnixPath.add_extension cs.cs_name "cmxs")]
+                     ["use_"^cs.cs_name]
+                 in
+
+                 let tag_t =
+                   if lib.lib_pack then
+                     let base_sources =
+                       OASISLibrary.source_unix_files
+                         ~ctxt:ctxt.ctxt
+                         (cs, bs, lib)
+                         (fun ufn ->
+                            OASISFileUtil.file_exists_case
+                              (OASISHostPath.of_unix ufn))
+                     in
+                     add_tags
+                       tag_t
+                       (List.rev_map
+                          (fun (base_fn, _) ->
+                             OASISUnixPath.add_extension base_fn "cmx")
+                          base_sources)
+                       ["for-pack("^String.capitalize cs.cs_name^")"]
+                   else
                      tag_t
-                     (List.filter
-                        (fun fn -> not (FilePath.UnixPath.is_current fn))
-                        (src_dirs @ src_internal_dirs))
-                     ["include"]
                  in
 
                  let ctxt, tag_t, myocamlbuild_t =
-                   bs_tags 
-                     pkg sct cs bs 
+                   bs_tags
+                     pkg sct cs bs
                      src_dirs
                      src_internal_dirs
                      target_lib
@@ -654,36 +703,57 @@ let add_ocamlbuild_files ctxt pkg =
                      myocamlbuild_t
                  in
 
-                 let myocamlbuild_t = 
-                   {myocamlbuild_t with 
-                        lib_ocaml = 
-                          (prepend_bs_path bs cs.cs_name, 
-                           List.filter 
-                             (fun fn -> not (FilePath.UnixPath.is_current fn))
+                 let myocamlbuild_t =
+                   {myocamlbuild_t with
+                        lib_ocaml =
+                          (cs.cs_name,
+                           List.filter
+                             (fun fn -> not (OASISUnixPath.is_current fn))
                              src_dirs) :: myocamlbuild_t.lib_ocaml}
                  in
 
                  let () =
                    if lib.lib_modules = [] then
-                     warning 
+                     warning
                        ~ctxt:ctxt.ctxt
                        (f_ "No exported module defined for library %s")
                        cs.cs_name;
                  in
 
                  let ctxt =
-                   (* Generate .mllib files *)
-                   let fn_base = 
-                     prepend_bs_path bs cs.cs_name
+                   (* Generate .mllib or .mlpack files *)
+                   let extension, not_extension =
+                     if lib.lib_pack then
+                       "mlpack", "mllib"
+                     else
+                       "mllib", "mlpack"
                    in
+                   let fn_base = prepend_bs_path bs cs.cs_name in
+                   let fn =
+                     OASISHostPath.add_extension fn_base extension
+                   in
+                   let not_fn =
+                     OASISHostPath.add_extension fn_base not_extension
+                   in
+                   let ctxt =
                      add_file
                        (template_make
-                          (FilePath.add_extension fn_base "mllib")
+                          fn
                           comment_ocamlbuild
                           []
                           (lib.lib_modules @ lib.lib_internal_modules)
                           [])
                        ctxt
+                   in
+                     {ctxt with
+                          other_actions =
+                            (fun ()->
+                               if OASISFileUtil.file_exists_case not_fn then
+                                 OASISMessage.error ~ctxt:ctxt.ctxt
+                                   (f_ "Conflicting file '%s' and '%s' \
+                                      exists, remove '%s'.")
+                                   fn not_fn not_fn)
+                          :: ctxt.other_actions}
                  in
 
                    ctxt, tag_t, myocamlbuild_t
@@ -692,41 +762,41 @@ let add_ocamlbuild_files ctxt pkg =
            | Executable (cs, bs, exec) as sct ->
                begin
                  (* Extract content for executables *)
-                 let src_dirs = 
-                   bs_paths bs [exec.exec_main_is]
+                 let src_dirs, src_internal_dirs =
+                   MapDirs.find (`Executable cs.cs_name) map_dirs
                  in
 
-                 let target_exec = 
+                 let target_exec =
                    let ext =
-                     match bs.bs_compiled_object with 
+                     match bs.bs_compiled_object with
                        | Best ->
                            "{native,byte}"
-                       | Byte -> 
+                       | Byte ->
                            "byte"
                        | Native ->
                            "native"
                    in
                      prepend_bs_path bs
-                       (FilePath.UnixPath.replace_extension 
+                       (OASISUnixPath.replace_extension
                           (exec.exec_main_is) ext)
                  in
 
-                 let tag_t = 
+                 let tag_t =
                    (Printf.sprintf "# Executable %s" cs.cs_name) :: tag_t
                  in
 
                  let ctxt, tag_t, myocamlbuild_t =
-                   bs_tags 
-                     pkg sct cs bs 
+                   bs_tags
+                     pkg sct cs bs
                      src_dirs
-                     []
+                     src_internal_dirs 
                      target_exec
                      ctxt
                      tag_t
                      myocamlbuild_t
                  in
 
-                 let tag_t = 
+                 let tag_t =
                    if exec.exec_custom then
                      add_tags tag_t [target_exec] ["custom"]
                    else
@@ -738,12 +808,12 @@ let add_ocamlbuild_files ctxt pkg =
 
            | Flag _ | SrcRepo _ | Test _ | Doc _ ->
                ctxt, tag_t, myocamlbuild_t)
-      (ctxt, [], {lib_ocaml = []; lib_c = []; flags = []})
+      (ctxt, [], {lib_ocaml = []; lib_c = []; flags = []; includes = []})
       pkg.sections
   in
 
   (* Filter duplicate and reverse content in tag_t *)
-  let tag_t = 
+  let tag_t =
     snd
       (List.fold_left
          (fun (prev_tag, acc) tag ->
@@ -765,41 +835,56 @@ let add_ocamlbuild_files ctxt pkg =
       lib_ocaml = List.rev myocamlbuild_t.lib_ocaml;
       lib_c     = List.rev myocamlbuild_t.lib_c;
       flags     = List.rev myocamlbuild_t.flags;
+      includes  = compute_includes map_dirs pkg;
     }
   in
 
-  let ctxt = 
+  let tag_t = 
+    "# Ignore VCS directories, you can use the same kind of rule outside "
+    ::
+    "# OASIS_START/STOP if you want to exclude directories that contains "
+    ::
+    "# useless stuff for the build process"
+    ::
+    add_tags
+      tag_t
+      ["_darcs"; ".git"; ".hg"; ".bzr"; "**/.svn"]
+      ["not_hygienic"; "-traverse"]
+  in
+
+
+  let ctxt =
     List.fold_left
       (fun ctxt tmpl -> add_file tmpl ctxt)
       ctxt
       [
         (* Generate _tags *)
-        template_make 
-          "_tags" 
-          comment_ocamlbuild 
+        template_make
+          "_tags"
+          comment_ocamlbuild
           []
           tag_t
           [];
 
         (* Generate myocamlbuild.ml *)
-        template_of_mlfile 
+        template_of_mlfile
          "myocamlbuild.ml"
           []
           [
-            OASISData.oasissyslight_ml; 
+            OASISData.oasissyslight_ml;
             BaseData.basesysenvironment_ml;
             OCamlbuildData.myocamlbuild_ml;
             "open Ocamlbuild_plugin;;";
             (
               Format.fprintf Format.str_formatter
                 "@[<hv2>let package_default =@ %a@,@];;"
-                (pp_odn ~opened_modules:["Ocamlbuild_plugin"]) 
+                (pp_odn ~opened_modules:["Ocamlbuild_plugin"])
                 (MyOCamlbuildBase.odn_of_t myocamlbuild_t);
               Format.flush_str_formatter ()
             );
             "";
             "let dispatch_default = \
-                   MyOCamlbuildBase.dispatch_default package_default;;"; 
+                   MyOCamlbuildBase.dispatch_default package_default;;";
             "";
           ]
           ["Ocamlbuild_plugin.dispatch dispatch_default;;"];
@@ -808,12 +893,12 @@ let add_ocamlbuild_files ctxt pkg =
 
     ctxt
 
-let qstrt_completion pkg = 
+let qstrt_completion pkg =
   fix_build_tools (ExternalTool "ocamlbuild") pkg
 
 let init () =
   let doit ctxt pkg =
-    let ctxt = 
+    let ctxt =
       add_ocamlbuild_files ctxt pkg
     in
 
