@@ -1,23 +1,23 @@
-(********************************************************************************)
-(*  OASIS: architecture for building OCaml libraries and applications           *)
-(*                                                                              *)
-(*  Copyright (C) 2008-2010, OCamlCore SARL                                     *)
-(*                                                                              *)
-(*  This library is free software; you can redistribute it and/or modify it     *)
-(*  under the terms of the GNU Lesser General Public License as published by    *)
-(*  the Free Software Foundation; either version 2.1 of the License, or (at     *)
-(*  your option) any later version, with the OCaml static compilation           *)
-(*  exception.                                                                  *)
-(*                                                                              *)
-(*  This library is distributed in the hope that it will be useful, but         *)
-(*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  *)
-(*  or FITNESS FOR A PARTICULAR PURPOSE. See the file COPYING for more          *)
-(*  details.                                                                    *)
-(*                                                                              *)
-(*  You should have received a copy of the GNU Lesser General Public License    *)
-(*  along with this library; if not, write to the Free Software Foundation,     *)
-(*  Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA               *)
-(********************************************************************************)
+(******************************************************************************)
+(* OASIS: architecture for building OCaml libraries and applications          *)
+(*                                                                            *)
+(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(*                                                                            *)
+(* This library is free software; you can redistribute it and/or modify it    *)
+(* under the terms of the GNU Lesser General Public License as published by   *)
+(* the Free Software Foundation; either version 2.1 of the License, or (at    *)
+(* your option) any later version, with the OCaml static compilation          *)
+(* exception.                                                                 *)
+(*                                                                            *)
+(* This library is distributed in the hope that it will be useful, but        *)
+(* WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY *)
+(* or FITNESS FOR A PARTICULAR PURPOSE. See the file COPYING for more         *)
+(* details.                                                                   *)
+(*                                                                            *)
+(* You should have received a copy of the GNU Lesser General Public License   *)
+(* along with this library; if not, write to the Free Software Foundation,    *)
+(* Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA              *)
+(******************************************************************************)
 
 (** Run full OASIS use cases
     @author Sylvain Le Gall
@@ -29,6 +29,12 @@ open TestCommon;;
 
 type filename = FilePath.filename;;
 
+let exec fn = 
+  if Sys.os_type = "Win32" then
+    fn^".exe"
+  else
+    fn
+
 let compare_filename =
   if Sys.os_type = "Win32" then
     (* Win32 FS is case insensitive *)
@@ -146,15 +152,89 @@ let tests =
       SetFile.empty
   in
 
+  let setup_ml = "setup.ml" in
+
   (* Assert with setup.ml *)
   let assert_run_setup ?exit_code ?extra_env args =
-    assert_command ?exit_code ?extra_env "ocaml" ("setup.ml" :: args)
+    (* Speed up for testing, compile setup.ml *)
+    let can_compile =
+      let chn = open_in setup_ml in
+      let hash_load = ref false in
+      let () =
+        try
+          while not !hash_load do
+            if OASISString.starts_with ~what:"#load" (input_line chn) then
+              hash_load := true
+          done
+        with End_of_file ->
+          ()
+      in
+        close_in chn;
+        not !hash_load
+    in
+    let setup_base = Filename.chop_extension setup_ml in
+    let setup_digest = setup_base ^ ".digest" in
+    let setup_exe = 
+      Filename.concat Filename.current_dir_name (exec setup_base)
+    in
+      if can_compile then
+        begin
+          let self_digest =
+            let chn = open_in setup_ml in
+            let digest = Digest.channel chn (in_channel_length chn) in
+              close_in chn;
+              digest
+          in
+          let pre_digest =
+            try
+              let chn = open_in setup_digest in
+              let digest =
+                try
+                  Digest.input chn 
+                with _ ->
+                  Digest.string ""
+              in
+                close_in chn;
+                digest
+            with _ ->
+              Digest.string ""
+          in
+          let clean ?(all=false) () =
+            List.iter
+              (fun fn -> try Sys.remove fn with _ -> ())
+              (
+                (if all then
+                   [setup_exe; setup_digest]
+                 else
+                   [])
+                @
+                [setup_base ^ ".cmi"; setup_base ^ ".cmo"]
+              )
+          in
+          if not (Sys.file_exists setup_exe) || self_digest <> pre_digest then
+            begin
+              match Sys.command ("ocamlfind ocamlc -o "^(exec "setup")^" setup.ml") with
+                | 0 ->
+                    (* Compilation succeed, update the digest *)
+                    let chn = open_out setup_digest in
+                      Digest.output chn self_digest;
+                      close_out chn;
+                      clean ()
+                | _ ->
+                    prerr_endline "E: Compilation of setup.ml doesn't succeed.";
+                    clean ~all:true ()
+            end
+        end;
+      if Sys.file_exists setup_exe then
+        assert_command ?exit_code ?extra_env setup_exe ("-info" :: "-debug" :: args)
+      else
+        assert_command ?exit_code ?extra_env "ocaml" (setup_ml :: "-info" :: "-debug" :: args)
   in
 
   (* Files always generated *)
   let oasis_std_files = 
     [
-      "setup.ml"; 
+      setup_ml; 
     ]
   in
 
@@ -226,9 +306,8 @@ let tests =
   let add_path nm dir = 
     nm,
     try 
-      Sys.getenv nm ^
-      (if Sys.os_type = "Win32" then ";" else ":")^ 
-      dir
+      FilePath.string_of_path
+        ((FilePath.path_of_string (Sys.getenv nm)) @ [dir])
     with Not_found ->
       dir
   in
@@ -347,15 +426,11 @@ let tests =
 
         try 
           let pkg_as_module =
-            let res =
-              Buffer.create (String.length pkg)
-            in
-              String.iter
-                (function 
-                   | '-' -> Buffer.add_char res '_'
-                   | c -> Buffer.add_char res c)
-                pkg;
-              Buffer.contents res
+            OASISString.replace_chars
+              (function
+                 | '-' -> '_'
+                 | c -> c)
+              pkg
           in
           let fn = 
             FilePath.concat srcdir ("test_"^pkg_as_module^".ml")
@@ -471,14 +546,57 @@ let tests =
   let long_test () =
     skip_if (not !long) "Long test"
   in
+
+  let filter_platform lst = 
+    let check_suff fn suff = Filename.check_suffix fn ("."^suff) in
+    List.fold_left 
+      (fun acc fn -> 
+         if (check_suff fn "cmx" || check_suff fn "cmxa") then
+           if !has_ocamlopt then
+             fn :: acc
+           else
+             acc
+         else if check_suff fn "cmxs" then
+           if !has_native_dynlink then
+             fn :: acc
+           else
+             acc
+         else if check_suff fn "a" then 
+           let fn =
+             if Sys.os_type = "Win32" then
+               (Filename.chop_extension fn) ^ ".lib"
+             else
+               fn
+           in
+             if OASISString.starts_with (Filename.basename fn) "lib" then
+               (* stubs library *)
+               fn :: acc
+             else if !has_ocamlopt then
+               (* library matching the .cmxa *)
+               fn :: acc
+             else
+               acc
+         else if check_suff fn "so" then
+           let fn = 
+             if Sys.os_type = "Win32" then
+               (Filename.chop_extension fn) ^ ".dll"
+             else
+               fn
+           in
+             fn :: acc
+         else 
+           fn :: acc)
+      [] lst
+  in
     
   let bracket_setup 
         ?(dev=false)
+        ?(dynamic=false)
         (srcdir, vars)
         f = 
     bracket
       (fun () ->
-         let (skip_cond, oasis_extra_files, installed_files, post_install_runs) = 
+         let (skip_cond, oasis_extra_files, _, post_install_runs) = 
            vars () 
          in
 
@@ -527,7 +645,7 @@ let tests =
            bak_lst)
 
       (fun (cur_dir, loc, pristine, bak_lst) ->
-         let (skip_cond, oasis_extra_files, installed_files, post_install_runs) = 
+         let (skip_cond, oasis_extra_files, _, post_install_runs) = 
            vars () 
          in
 
@@ -549,11 +667,78 @@ let tests =
 
          (* Create build system using OASIS *)
          let () = 
-           assert_oasis_cli 
-             (if dev then 
-                ["setup-dev"; "-real-oasis"; "-only-setup"] 
-              else 
-                ["setup"]);
+           assert_oasis_cli
+             ("setup" ::
+              (if dev then
+                 ["-real-oasis"; "-setup-update";
+                  if dynamic then "dynamic" else "weak"]
+               else
+                 []));
+
+           if Sys.file_exists "_tags" && !dbug then
+             begin
+               let chn = open_in "_tags" in
+                 print_endline "file _tags";
+                 try 
+                   while true do 
+                     print_endline (input_line chn)
+                   done
+                 with End_of_file ->
+                   close_in chn
+             end;
+
+           (* Fix #require in dynamic *)
+           if dynamic then
+             begin
+               let chn = open_in "setup.ml" in
+               let lst = ref [] in
+               let mkload lst =
+                 let cma =
+                   Filename.concat
+                     cur_dir
+                     (FilePath.make_filename (".." :: "_build" :: "src" :: lst))
+                 in
+                 Printf.sprintf
+                   "#load %S;;\n#directory %S;;" cma (Filename.dirname cma)
+               in
+               let () =
+                 try
+                   while true do
+                     let line = input_line chn in
+                       if OASISString.starts_with
+                            ~what:"#require \"oasis.dynrun\";;" line then
+                         begin
+                           lst :=
+                           List.rev_append
+                             ("#require \"unix\";;" ::
+                              "#require \"odn\";;" ::
+                              (List.map mkload
+                                 [["oasis"; "oasis.cma"];
+                                  ["base"; "base.cma"];
+                                  ["builtin-plugins.cma"];
+                                  ["dynrun"; "dynrun.cma"]]))
+                             !lst
+                         end
+                       else
+                         lst := line :: !lst
+                   done
+                 with End_of_file ->
+                   close_in chn
+               in
+               let chn = open_out "setup.ml" in
+               let () = 
+                 if !dbug then
+                   print_endline "file setup.ml:";
+                 List.iter
+                   (fun line ->
+                      if !dbug then
+                        print_endline line;
+                      output_string chn (line^"\n"))
+                   (List.rev !lst);
+                 close_out chn
+               in
+                 ()
+             end;
 
            (* Check generated files *)
            OUnitSetFile.assert_equal 
@@ -563,6 +748,44 @@ let tests =
              (all_files_cwd ())
          in
 
+        (* If a _tags file exists, try to test its content. *)
+        let () =
+          if Sys.file_exists "_tags" then
+            let chn = open_in "_tags" in
+            let lineno = ref 0 in
+              try
+                while true do
+                  let line = input_line chn in
+                  let () = incr lineno in
+                    try
+                      let _ = "(*" in
+                      let substr = Pcre.exec ~pat:"^\\s*\"(.*)\"\\s*:" line in
+                      let fn = Pcre.get_substring substr 1 in
+                        if List.mem fn
+                             [".git"; ".bzr"; ".hg"; "_darcs"] ||
+                           List.mem (FilePath.get_extension fn)
+                             ["byte"; "native"; "lib"; "dll"; "a"; "so"; "cmxs"] then
+                          ()
+                        else if FilePath.get_extension fn = "cmx" then
+                          begin
+                            let fn_ml = FilePath.replace_extension fn "ml" in
+                              if not  (Sys.file_exists fn_ml) then
+                                assert_failure
+                                  (Printf.sprintf
+                                     "source file '%s' doesn't exist" fn_ml)
+                          end
+                        else if not (Sys.file_exists fn) then
+                          assert_failure
+                            (Printf.sprintf "file '%s' doesn't exist in _tags file line %d (%S)" 
+                               fn !lineno line)
+                    with Not_found ->
+                      (* TODO: handle ocamlbuild wildcard *)
+                      ()
+                done
+              with End_of_file ->
+                close_in chn
+        in
+
            (* Run the main function *)
            f (cur_dir, loc, pristine, expected_post_oasis_files);
 
@@ -628,29 +851,14 @@ let tests =
       )
   in
 
-  (* Run devel test *)
-  let test_of_vector_dev (a, f)  =
-    "dev" >::
-    bracket_setup ~dev:true 
-      (a, 
-       (fun () -> 
-          let (skip_cond, _, d, e) = 
-            f ()
-          in
-             (fun () -> skip_cond (); long_test ()),
-             ["setup.ml"], d, e))
-      (fun _ ->
-         assert_run_setup ["-all"];
-         assert_run_setup ["-distclean"])
-  in
-
   (* Run short test *)
   let test_of_vector_short ?extra_env e =
     "ocaml setup.ml -all" >::
     bracket_setup e
       (fun _ -> 
          assert_run_setup ?extra_env ["-all"];
-         assert_run_setup ?extra_env ["-distclean"])
+         assert_run_setup ?extra_env ["-distclean"];
+         rm ~force:Force [exec "setup"; "setup.digest"])
   in
 
   (* Run standard test *)
@@ -669,10 +877,11 @@ let tests =
              (fun st e -> SetFile.add e st)
              SetFile.empty
              (* Compute all file that should have been installed *)
-             (List.fold_left
-                (fun acc f -> f loc acc)
-                []
-                installed_files)
+             (filter_platform
+               (List.fold_left
+                  (fun acc f -> f loc acc)
+                  []
+                  installed_files))
          in
 
          (* Run configure target *)
@@ -689,6 +898,55 @@ let tests =
              (Sys.file_exists "setup.data")
          in
 
+         (* Extract ocamlbuild flags and check that they are correct. *)
+         let () = 
+           if Sys.file_exists "myocamlbuild.ml" then
+             let documentation_output = 
+               let buf = Buffer.create 16000 in
+               OUnit.assert_command 
+                 ~foutput:(Stream.iter (Buffer.add_char buf))
+                 "ocamlbuild" ["-documentation"];
+               Buffer.contents buf
+             in
+             let lst = OASISUtils.split_newline documentation_output in
+             let rst = ref SetString.empty in
+             let () =
+               List.iter 
+                 (fun line ->
+                    try
+                      let _ = "(*" in
+                      let substr = Pcre.exec ~pat:"flag {\\. (.*) \\.}" line in
+                      let lst = 
+                        Pcre.split ~pat:"\\s*,\\s*" 
+                          (Pcre.get_substring substr 1)
+                      in
+                        rst :=
+                        List.fold_left
+                          (fun st e -> SetString.add e st)
+                          !rst
+                          lst
+                    with Not_found ->
+                      ())
+                 lst
+             in
+               if !rst = SetString.empty then
+                 assert_failure ("Set of flags should not be empty.");
+               SetString.iter 
+                 (fun flag ->
+                    String.iter 
+                      (function
+                         | 'A'..'Z' | 'a'..'z' | '0'..'9' 
+                         | '_' | '.' | ':' | '-' | '(' | ')' -> 
+                             ()
+                         | c ->
+                             assert_failure 
+                               (Printf.sprintf 
+                                  "flag %S contains %C which is illegal."
+                                  flag c))
+                      flag)
+                 !rst
+         in
+
          (* Run build target *)
          let () = 
            assert_run_setup ["-build"]
@@ -809,6 +1067,7 @@ let tests =
          (* Run clean target *)
            assert_run_setup ["-clean"];
            assert_run_setup ["-distclean"];
+           rm ~force:Force [exec "setup"; "setup.digest"];
 
            (* Check that only OASIS generated files remain *)
            OUnitSetFile.assert_equal
@@ -825,7 +1084,6 @@ let tests =
     [
       test_of_vector_std e;
       test_of_vector_short e;
-      test_of_vector_dev e;
     ]
   in
 
@@ -845,33 +1103,25 @@ let tests =
               "src/simplelibext/simplelibext.odocl";
             ],
             [
-              (in_ocaml_library "simplelib") 
+              in_ocaml_library "simplelib"
                 ["simplelib.cma"; 
                  "Foo.cmi"; "Foo.ml"; 
                  "Bar.cmi"; "Bar.ml"; 
-                 "META"];
-              conditional 
-                !has_ocamlopt
-                (in_ocaml_library "simplelib" 
-                   ["simplelib.cmxa"; 
-                    if Sys.os_type = "Win32" then
-                      "simplelib.lib"
-                    else
-                      "simplelib.a"]);
+                 "META";
+                 "simplelib.cmxa"; 
+                 "simplelib.cmxs";
+                 "Foo.cmx"; "Bar.cmx";
+                 "simplelib.a"];
 
               in_ocaml_library "simplelibext"
                 ["simplelibext.cma"; 
                  "FooExt.cmi"; "FooExt.ml"; 
                  "BarExt.cmi"; "BarExt.ml"; 
-                 "META"];
-              conditional
-                !has_ocamlopt
-                (in_ocaml_library "simplelibext"
-                   ["simplelibext.cmxa"; 
-                    if Sys.os_type = "Win32" then 
-                      "simplelibext.lib"
-                    else
-                      "simplelibext.a"]);
+                 "META";
+                 "simplelibext.cmxa"; 
+                 "simplelibext.cmxs";
+                 "FooExt.cmx"; "BarExt.cmx";
+                 "simplelibext.a"];
 
               api_ref_html "simplelib"
                 ["Foo"; "Bar"];
@@ -897,21 +1147,33 @@ let tests =
                 ["simplelib.cma"; 
                  "foo.cmi"; "foo.mli"; 
                  "bar.cmi"; "bar.mli"; 
-                 "META"];
-              conditional 
-                !has_ocamlopt
-                (in_ocaml_library "simplelib"
-                   ["simplelib.cmxa"; 
-                    if Sys.os_type = "Win32" then
-                      "simplelib.lib"
-                    else
-                      "simplelib.a"]);
+                 "META";
+                 "simplelib.cmxa"; 
+                 "simplelib.cmxs";
+                 "foo.cmx"; "bar.cmx";
+                 "simplelib.a"];
 
               api_ref_html "simplelib"
                 ["Bar"; "Foo"];
             ],
             []);
 
+         (* Packed library *)
+         "../examples/packedlib", 
+         (fun () -> 
+            long_test,
+            oasis_ocamlbuild_files @ 
+              [ "src/packedlib.mlpack"; "src/META" ],
+            [
+              in_ocaml_library "packedlib" 
+                ["packedlib.cma"; "packedlib.cmi";
+                 "foo.mli"; "bar.mli"; "Baz.ml"; "META";
+                 "packedlib.cmxa"; "packedlib.cmxs";
+                 "packedlib.a"];
+            ],
+            [
+              try_installed_library "packedlib" ["Packedlib.Foo"]
+            ]);
 
          (* Complete library with findlib package to check *)
          "../examples/findlib",
@@ -921,7 +1183,6 @@ let tests =
             [],
             []);
 
-
          (* Complete library with custom build system *)
          "../examples/custom", 
          (fun () -> 
@@ -936,51 +1197,29 @@ let tests =
             ],
             []);
 
-
          (* Library/executable using C files *)
          "../examples/with-c",
          (fun () -> 
             long_test,
             [
               "src/META"; 
-              "src/libtest-with-c-custom.clib"; 
-              "src/libtest-with-c-native.clib";
-              "src/libtest-with-c.clib";
-              "src/libwith-c.clib";
+              "src/libtest-with-c-custom_stubs.clib";
+              "src/libtest-with-c-native_stubs.clib";
+              "src/libtest-with-c_stubs.clib";
+              "src/libwith-c_stubs.clib";
               "src/with-c.mllib";
               "src/with-c.odocl";
             ] @ oasis_ocamlbuild_files,
             [
-              in_bin 
-                (if Sys.os_type = "Win32" then
-                   ["test-with-c.exe"; "test-with-c-custom.exe"]
-                 else
-                   ["test-with-c"; "test-with-c-custom"]);
+              in_bin [exec "test-with-c"; exec "test-with-c-custom"];
               conditional
                 !has_ocamlopt
-                (in_bin [if Sys.os_type = "Win32" then
-                           "test-with-c-native.exe"
-                         else
-                           "test-with-c-native"]);
-              in_library [if Sys.os_type = "Win32" then
-                            "with-c/dlltest-with-c.dll"
-                          else
-                            "with-c/dlltest-with-c.so"];
+                (in_bin [exec "test-with-c-native"]);
+              in_library ["with-c/dlltest-with-c_stubs.so"];
               in_ocaml_library "with-c"
-                ["A.cmi"; "A.ml"; "META"; "with-c.cma"]; 
-              in_ocaml_library "with-c"
-                (if Sys.os_type = "Win32" then
-                   ["libwith-c.lib"; "dllwith-c.dll"]
-                 else
-                   ["libwith-c.a"; "dllwith-c.so"]);
-              conditional
-                !has_ocamlopt
-                (in_ocaml_library "with-c" 
-                   [if Sys.os_type = "Win32" then
-                      "with-c.lib"
-                    else
-                      "with-c.a"; 
-                    "with-c.cmxa"]);
+                ["A.cmi"; "A.ml"; "META"; "with-c.cma"; 
+                 "libwith-c_stubs.a"; "dllwith-c_stubs.so";
+                 "with-c.a"; "A.cmx"; "with-c.cmxa"; "with-c.cmxs"];
 
               api_ref_html "with-c"
                 ["A"];
@@ -1010,10 +1249,7 @@ let tests =
               "src/test.odocl";
             ] @ oasis_ocamlbuild_files,
             [
-              in_bin [if Sys.os_type = "Win32" then
-                        "test.exe"
-                      else
-                        "test"];
+              in_bin [exec "test"];
               in_ocaml_library "test"
                 [
                   "test.ml"; "test.cmi"; "META"; "test.cma";
@@ -1052,15 +1288,9 @@ let tests =
              in_ocaml_library "test" 
                ["META"; "test.cma"; "pa_test.cma";
                 "A.ml"; "A.cmi"; "B.ml"; "B.cmi";
-                "pa_test.ml"; "pa_test.cmi"];
-             conditional 
-               !has_ocamlopt
-               (in_ocaml_library "test"
-                  ["test.cmxa"; 
-                   if Sys.os_type = "Win32" then
-                     "test.lib"
-                   else
-                     "test.a"]);
+                "pa_test.ml"; "pa_test.cmi";
+                "test.cmxa"; "test.cmxs"; "A.cmx"; "B.cmx";
+                "test.a"];
 
              api_ref_html "test"
                ["A"; "B"];
@@ -1108,19 +1338,10 @@ let tests =
             ] @ oasis_ocamlbuild_files,
             [
               in_ocaml_library "with-a"
-                ["META"; "A.ml"; "A.cmi"; "with-a.cma"];
-              conditional
-                !has_ocamlopt
-                (in_ocaml_library "with-a"
-                   ["with-a.cmxa"; 
-                    if Sys.os_type = "Win32" then
-                      "with-a.lib"
-                    else
-                      "with-a.a"]);
-              in_bin [if Sys.os_type = "Win32" then
-                        "test-with-a.exe"
-                      else
-                        "test-with-a"];
+                ["META"; "A.ml"; "A.cmi"; "with-a.cma";
+                 "A.cmx"; "with-a.cmxa"; "with-a.cmxs"; 
+                 "with-a.a"];
+              in_bin [exec "test-with-a"];
               api_ref_html "with-a" ["A"];
             ],
             [
@@ -1150,7 +1371,7 @@ let tests =
             ["src/META"; 
              "Makefile"; 
              "configure"; 
-             "src/libtest_oasis_c_dependency.clib"; 
+             "src/libtest_oasis_c_dependency_stubs.clib";
              "src/test_oasis_c_dependency.mllib"] @ oasis_ocamlbuild_files,
             [],
             []);
@@ -1219,10 +1440,64 @@ let tests =
             "src/test.mllib" :: "src/META" :: oasis_ocamlbuild_files,
             [in_ocaml_library "test" ["META"; "foo.cmi"; "test.cma"]],
             []);
+
+         "data/bug982",
+         (fun () ->
+            (fun () ->
+              long_test ();
+              skip_if (Sys.os_type = "Win32") "UNIX only test"),
+            oasis_ocamlbuild_files,
+            [],
+            []);
+
+         "data/bug823",
+         (fun () ->
+            (fun () -> 
+               long_test ();
+               skip_if (Sys.os_type = "Win32") "UNIX test"),
+            oasis_ocamlbuild_files,
+            [],
+            []);
+
+         "data/bugClib",
+         (fun () ->
+            long_test,
+            "META" :: "mylib.mlpack" :: "libmylib_stubs.clib" ::
+            oasis_ocamlbuild_files,
+            [in_ocaml_library "mylib"
+               ["META"; "dllmylib_stubs.so";
+                "foo.ml"; "mylib.cma"; "mylib.cmi";
+                "mylib.cmxa"; "mylib.cmxs"; 
+                "mylib.a"; "libmylib_stubs.a"]],
+            [
+              try_installed_library "mylib" ["Mylib.Foo"; "Mylib.Bar"]
+            ]);
+
+          "data/bug791",
+          (fun () ->
+             long_test,
+             "src/testA.mllib" :: oasis_ocamlbuild_files,
+             [],
+             []);
        ]
     )
     @
-    ["TEMP=a b">::
+    ["data/bug938">::
+     bracket_setup 
+       ("data/bug938",
+        fun () ->
+          (fun () -> skip_if (Sys.os_type = "Win32") "UNIX test"),
+          oasis_ocamlbuild_files,
+          [],
+          [])
+       (* Run test *)
+       (fun _ ->
+          (* Run configure target *)
+          assert_run_setup 
+            ["-configure"; "--enable-all"; "--disable-over"];
+          rm ["setup.data"; exec "setup"; "setup.digest"]);
+
+     "TEMP=a b">::
      bracket
        (fun () ->
           let dn = readlink "a b" in
@@ -1243,9 +1518,141 @@ let tests =
                              else
                                "TMPDIR", dn]
                  ["-configure"];
-               rm ["setup.data"])
+               rm ["setup.data"; exec "setup"; "setup.digest"])
             ())
        (fun dn ->
-          rm ~recurse:true [dn])
+          rm ~recurse:true [dn]);
+
+     "setup with dev mode (weak)">::
+      bracket
+        ignore
+        (fun () ->
+           skip_if (Sys.os_type = "Win32") "UNIX test";
+           cp ["data/dev/_oasis.v1"] "data/dev/_oasis";
+           bracket_setup
+             ~dev:true
+             ("data/dev",
+              fun () ->
+                ignore,
+                oasis_ocamlbuild_files,
+                [],
+                [])
+             (* Run test *)
+             (fun _ ->
+                assert_run_setup ["-all"];
+                assert_bool
+                  "Library .cma not created."
+                  (not (Sys.file_exists "_build/mylib.cma"));
+                cp ["_oasis.v2"] "_oasis";
+                assert_run_setup ["-all"];
+                assert_bool
+                  "Library .cma created."
+                  (Sys.file_exists "_build/mylib.cma");
+                assert_run_setup ["-distclean"];
+                rm ["META"; "mylib.mllib"; exec "setup"; "setup.digest"];
+                cp ["_oasis.v1"] "_oasis")
+             ())
+        (fun () ->
+           rm ["data/dev/_oasis"]);
+
+    "setup with dev mode (light)">::
+    bracket
+      ignore
+      (fun () ->
+           cp ["data/dev/_oasis.v2"] "data/dev/_oasis";
+           bracket_setup
+             ~dev:true ~dynamic:true
+             ("data/dev",
+              fun () ->
+                ignore,
+                ["setup.ml"],
+                [],
+                [])
+             (* Run test *)
+             (fun _ ->
+                assert_bool
+                  "setup.ml is smaller than 2kB"
+                  (let chn = open_in "setup.ml" in
+                     try 
+                       let size = in_channel_length chn in
+                         close_in chn;
+                         size < 2048 (* 2kB *)
+                     with e ->
+                       close_in chn;
+                       raise e);
+                assert_run_setup ["-all"];
+                assert_bool
+                  "Library .cma created."
+                  (Sys.file_exists "_build/mylib.cma");
+                assert_run_setup ["-distclean"];
+                rm ["META"; "mylib.mllib"; exec "setup"; "setup.digest"])
+             ())
+      (fun () ->
+         rm ["data/dev/_oasis"]);
+
+     "setup with no dev mode">::
+     bracket
+       ignore
+       (fun () ->
+          cp ["data/dev/_oasis.v1"] "data/dev/_oasis";
+          bracket_setup
+            ("data/dev",
+             fun () ->
+               ignore,
+               oasis_ocamlbuild_files,
+               [],
+               [])
+            (* Run test *)
+            (fun _ ->
+               assert_run_setup ["-all"];
+               assert_bool
+                 "Library .cma not created."
+                 (not (Sys.file_exists "_build/mylib.cma"));
+               cp ["_oasis.v2"] "_oasis";
+               assert_run_setup ["-all"];
+               assert_bool
+                 "Library .cma still not created."
+                 (not (Sys.file_exists "_build/mylib.cma"));
+               assert_run_setup ["-distclean"];
+               rm ["META"; "mylib.mllib"; exec "setup"; "setup.digest"];
+               cp ["_oasis.v1"] "_oasis")
+            ())
+       (fun () ->
+          rm ["data/dev/_oasis"]);
+
+    "ver0.3">::
+    bracket_setup
+      ("data/ver0.3",
+       fun () ->
+         ignore,
+         oasis_ocamlbuild_files,
+         [],
+         [])
+      (* Run test *)
+      (fun _ ->
+         assert_run_setup ["-configure"];
+         assert_run_setup ["-test"];
+         assert_bool
+           "test not run."
+           (not (Sys.file_exists "test-done"));
+         assert_run_setup ["-doc"];
+         assert_bool
+           "doc done."
+           (Sys.file_exists "doc-done");
+         assert_run_setup ["-distclean"];
+         rm ["test-done"; "doc-done"];
+
+
+         assert_run_setup ["-configure"; "--enable-tests"; "--disable-docs"];
+         assert_run_setup ["-test"];
+         assert_bool
+           "test run."
+           (Sys.file_exists "test-done");
+         assert_run_setup ["-doc"];
+         assert_bool
+           "doc not done."
+           (not (Sys.file_exists "doc-done"));
+         assert_run_setup ["-distclean"];
+         rm ["test-done"; "doc-done"; exec "setup"; "setup.digest"])
     ]
 ;;
